###################################################################################
# Psychological Methods : SEM with PISA2009 Reading
###################################################################################
setwd("D:/2017/EvalReview/Reading_Result2_16_RuninMay222017")

# Load packages --------------------------------------------------------#
install.packages("lavaan")

## manually install the BMASEM package
install.packages("BMASEM_2.1.tar.gz", repos = NULL, type="source")

library(BMASEM)
require(lavaan)

PISASEM.MAD <- read.csv("MAD.csv")
PISASEM.PTD <- read.csv("PTD.csv")

summary(PISASEM.MAD)


## Set arguments for BMASEM
ex.vec <- c("ESCS", "GEND", "NATI")
med.vec <- c("MEMO", "ELAB", "CSTR")
med1.vec <- c("JOYR")
last.en <- "RCOM"

from.vars = c(rep(ex.vec, length(med.vec)), ex.vec, med.vec, med1.vec)
to.vars = c(rep(med.vec, each=length(ex.vec)), rep(med1.vec, length(ex.vec)+length(med.vec)), last.en)

# the cut-off score for occam's window, OR
## default : 20 
## with OR=20, only one model was selected 
## with OR=100, 

# link for the starting model
## defalut : NA (randomly selected)
start.link <- c(1,1,0, 1,1,1, 1,1,1, 0,0,0,1,1,1, 1) # the initial model

# call BMASEM (from.vars, to.vars, data, OR=20, start.link=NA)
bma <- BMASEM (from.vars, to.vars, data=PISASEM.MAD, start.link=start.link, OR=100)

summary(bma)
# check BIC
bics <- read.csv("Model_BIC.csv")
bics.1 = which(bics$BIC != 0) #599
bics.2 = bics[bics.1, ]
bics.3 = bics.2[order(bics.2$BIC), ]
head(bics.3)
bics.3[2, 2] - bics.3[1, 2]

summary(bma)
str(bma)

sink("BMASEM_summary_1028.out")
summary(bma)
str(bma)
sink()


# bma$label
bma.models = data.frame(cbind(Models=bma$model, PMP=round(bma$postprob,2), BIC=round(bma$bic,2)))
write.csv(bma.models, file="bmaModels_1028.csv", row.names=F)


# # str(bma)
# parameter=bma$parameter
# mean=bma$postmean
# # sd=bma$postsd
# # prob=c(100.0,bma$probne0)
# # bma.out <- data.frame(round(cbind(mean, sd, prob), 2))
# 
# bma.out <- data.frame(cbind(parameter, round(cbind(mean), 2)))
# write.csv(bma.out, file="bmaResult.csv", row.names=F)


# str(bma)
mean=bma$postmean
sd=bma$postsd
prob=bma$probne0
bma.out <- data.frame(round(cbind(mean, sd, prob), 2))
bma.out <- cbind(parameter=bma$parameter, bma.out)
write.csv(bma.out, file="bmaResult_1028.csv", row.names=F)

####################################################################
# Predict the final y 
####################################################################
pred.print <- function(y.hat, y.obs, title, ...)
{
  bma.lwr = y.hat$lwr
  bma.upr = y.hat$upr
  print(paste0("#-- 90% Coverage : ", title, " --#"))      
  print(1- sum(y.obs < bma.lwr | y.obs > bma.upr) / length(y.obs))
  print("#---------------------------#")      
}

new.y<-PISASEM.PTD[, "RCOM"]
head(PISASEM.PTD)


sink("BMASEM_Predict_1028.out")

## Predict BMASEM
bma.y.hat <- predict(bma, PISASEM.PTD)
pred.print(bma.y.hat, new.y,  "BMASEM")

## Predict the best model in BMASEM
best.y.hat <- predict(bma, PISASEM.PTD, bma.model=1)
pred.print(best.y.hat, new.y, "the best model")

sink()
