#---------------------------------------------------#
#------------- BAYESIAN LOGISTIC REGRESSION---------#
# install.packages("MCMCpack")
# install.packages("BMA")
library(MCMCpack)
library(BMA)

#Multiple Logistic Regression Analysis :
ECLS_K <- read.csv(file.choose(),header=T)
summary(ECLS_K) #no missing
ECLS_KLogReg <- ECLS_K

head(ECLS_KLogReg)
nrow(ECLS_KLogReg)
# ECLS_KLogReg<-na.omit(ECLS_KLogReg)
# nrow(ECLS_KLogReg)

setwd("D:/PsychM/BMA_Logistic")
# write.csv(ECLS_KLogReg, file="ECLS_K_Logi.csv", row.names=F)
# ECLS_KLogReg <- read.csv(file="ECLS_K_Logi.csv")

# Split data into two data set
data.index = sample(1:nrow(ECLS_KLogReg), size=nrow(ECLS_KLogReg)/2, replace=F)
ECLS_KLog.MAD <- ECLS_KLogReg[data.index, ] # data for model averaging 
ECLS_KLog.PTD <- ECLS_KLogReg[-data.index, ] # data for predictive testing
# 
write.csv(ECLS_KLog.MAD, file="MAD.csv", row.names=F)
write.csv(ECLS_KLog.PTD, file="PTD.csv", row.names=F)

ECLS_KLog.MAD <- read.csv(file="MAD.csv")
ECLS_KLog.PTD <- read.csv(file="PTD.csv")

#### Run model ######################################## 
fmla <- as.formula(fullday ~GENDER+RACE+P1HMEMP+ P1AGEENT+ P1AGEFRS
                   + P1PRIMNW+ P1LANGUG+ P1NUMSIB+ P1HFAMIL+ WKHEARLY
                   + WKNUMPRK+ WKSESL + P1EXPECT+ P1READBO)

## BAYESIAN MODEL AVERAGING-------------#
bma <- bic.glm(fmla, data=ECLS_KLog.MAD, glm.family="binomial", strict=FALSE, OR=20) 

pdf("Log_PMBMA_posterior.pdf")
plot(bma) # Plots of BMA posterior distributions
imageplot.bma(bma) # The image plot shows which predictors are included in each model
dev.off()

sink("BMASummary.out")
summary(bma)
sink()

# bma$label
bma.models = data.frame(cbind(PMP=round(bma$postprob,2), BIC=round(bma$bic,2), Label=bma$label))
Models <- paste0("M", 1:dim(bma.models)[1])
bma.models <- cbind(Models, bma.models)
write.csv(bma.models, file="bmaModels.csv", row.names=F)

# str(bma)
mean=bma$postmean
sd=bma$postsd
prob=c(100.0,bma$probne0)
bma.out <- data.frame(round(cbind(mean, sd, prob), 2))
write.csv(bma.out, file="bmaResult.csv", row.names=F)


## B-Model(the best model in BMA)-------#
str(bma)
# to check the coeff. and se 
bma$mle[1, ]
bma$se[1, ]

bma1.label <- unlist(strsplit(bma$label[1], split="[,]"))
data.glm <- ECLS_KLog.MAD[, c("fullday", bma1.label)]
head(data.glm)

b.glm <- glm(fullday~., data=data.glm, family="binomial")
summary(b.glm)

#---------Bayesian Logistic Regression-------------#
FullModel <- MCMClogit(fmla,data=ECLS_KLog.MAD, marginal.likelihood="Laplace", 
                       mcmc=100000, b0=0, B0=(.01), burnin=5000, thin=50,tune=0.25)

pdf("Log_Bayesian_posterior.pdf")
plot(FullModel) # Produces the convergence plots and the posterior densities
dev.off()

sink("MCMCsummary.out")
summary(FullModel)
sink()

p.name = unlist(attr(FullModel, "dimnames"))
mean = summary(FullModel)$statistics[, 1]
sd = summary(FullModel)$statistics[, 2]
q.2.5 = summary(FullModel)$quantiles[, 1]
q.97.5 = summary(FullModel)$quantiles[, 5]

Bayes.result = data.frame(p.name, mean, sd, q.2.5, q.97.5)
write.csv(Bayes.result, file="Bayes_Result.csv", row.names=F)

## Frequentist : Logistic Regression model ------#
s.glm <- glm(fmla, data=ECLS_KLog.MAD, family="binomial")
summary(s.glm)

##########################################################################
# Brier Score: Predictive performance for the test data -----------------#
##########################################################################
# Function for Brier Score 
## the sum of squares of the difference between new y values 
### and the predicted probability 
Brierscore<-function(ps,new.y)
{
  Brierscore=sum((new.y-ps)^2)
  return(Brierscore)
}

# with the test data
new.glm=glm(fmla, data=ECLS_KLog.PTD, family="binomial")
new.X=model.matrix(new.glm)  # create a design matrix of covariates
new.y <- ECLS_KLog.PTD$fullday

## BAYESIAN MODEL AVERAGING-----------------#
bma.temp=new.X %*% bma$postmean
bma.ps=exp(bma.temp)/(1+exp(bma.temp))

## B-Model(the best model in BMA)-----------#
b.glm.ps = predict.glm(b.glm, newdata=ECLS_KLog.PTD, type="response")

## Bayesian Logistic Regression-------------#
# ! Not running with my computer 
# bayes.ps.all=matrix(rep(0,nrow(FullModel)*nrow(ECLS_KLog.PTD)),nrow=nrow(ECLS_KLog.PTD))
# for(i in 1:nrow(FullModel))    # ith simulated set of beta => one set of ps
# {
#   temp=new.X %*% FullModel[i,];
#   bayes.ps.all[,i]=exp(temp)/(1+exp(temp));
# }
# 
# # Posterior Mean of Predicted Bayesian Logistic regression score for Each Subject
# bayes.ps <- apply(bayes.ps.all, 1, mean)

bayes.temp=new.X %*% summary(FullModel)$statistics[, 1]
bayes.ps=exp(bayes.temp)/(1+exp(bayes.temp));

## Frequentist Logistic Regression
s.glm.ps = predict.glm(s.glm,newdata=ECLS_KLog.PTD,type="response")

## Get Brier Score -----------------------
sink("Brier_Score.out")
print(paste("BMA", round(Brierscore(bma.ps, new.y),2), sep=" : "))
print(paste("Best", round(Brierscore(b.glm.ps, new.y),2), sep=" : "))
print(paste("Bayesian", round(Brierscore(bayes.ps, new.y),2), sep=" : "))
print(paste("Frequentist", round(Brierscore(s.glm.ps, new.y),2), sep=" : "))
sink()


sink("Brier_Score_mean.out")
print(paste("BMA", round(Brierscore(bma.ps, new.y)/length(new.y),2), sep=" : "))
print(paste("Best", round(Brierscore(b.glm.ps, new.y)/length(new.y),2), sep=" : "))
print(paste("Bayesian", round(Brierscore(bayes.ps, new.y)/length(new.y),2), sep=" : "))
print(paste("Frequentist", round(Brierscore(s.glm.ps, new.y)/length(new.y),2), sep=" : "))
sink()