#---------- EXAMPLE 6:1: BAYESIAN MULTIPLE REGRESSION WITH NON-INFORMATIVE PRIORS ----------#
# install.packages("MCMCpack")
# install.packages("BMA")
# install.packages("coda")
library(MCMCpack)
library(coda)
library(BMA)

#  Read in data for the full data
setwd("D:/PsychM/BMA_Regression")

#Multiple Linear Regression Analysis :
PISA09Reg <- read.csv(file.choose(),header=T)
summary(PISA09Reg) 
#check missings

PISA09Reg_1 <- PISA09Reg

head(PISA09Reg_1)
nrow(PISA09Reg_1)

# Split data into two data set
data.index = sample(1:nrow(PISA09Reg_1), size=nrow(PISA09Reg_1)/2, replace=F)
PISA09Reg_1.MAD <- PISA09Reg_1[data.index, ] # data for model averaging 
PISA09Reg_1.PTD <- PISA09Reg_1[-data.index, ] # data for predictive testing
# 
write.csv(PISA09Reg_1.MAD, file="MAD.csv", row.names=F)
write.csv(PISA09Reg_1.PTD, file="PTD.csv", row.names=F)

### Read files
PISA09Reg.MAD= read.csv("MAD.csv")
PISA09Reg.PTD= read.csv("PTD.csv")

# for Prediction : information for the traing data set
N=nrow(PISA09Reg.MAD)
X = cbind(const=1, data.matrix(PISA09Reg.MAD[, -1], rownames.force = NA))
y = PISA09Reg.MAD$RCOM

# Function for Bayesian Predictive interval
## the results from this function are narrower than ones from predict.lm()
predict.PMBMA <- function(coef.vec, newdata, level=.90, ...)
{
  coef.vec = matrix(as.vector(coef.vec), ncol=1)
  
  new.X <- newdata[, -1]

  t.df <- N - ncol(X) #from the training data set
  t.p <- level + (1-level)/2
  t.value <-qt(t.p, t.df)

  res=as.vector(y-X %*% coef.vec )
  see=sum(res^2)
  mse=see/t.df
  
  predict.interval <- function(x.temp, ...)
  {
    x=t(cbind(1, x.temp))
    y.hat <- t(x) %*% coef.vec

    r.term = (1 + t(x) %*% solve(t(X)%*%X) %*% x )
    r.term.1 = sqrt(mse * r.term )
    
    t.lwr <- y.hat - t.value * r.term.1
    t.upr <- y.hat + t.value * r.term.1
    
    return(cbind(fit=y.hat, lwr=t.lwr, upr=t.upr))
  }

  result = NULL
  for (i in 1:N)
  {
    result = rbind(result, predict.interval(new.X[i, ]))
  }
  result = data.frame(result)
  names(result) = c("fit", "lwr", "upr")
  return(result)
}


#### Run model ######################################## 
## BAYESIAN MODEL AVERAGING-------------#
attach(PISA09Reg.MAD)
bma=bicreg(cbind(GEND, NATI,  SLAN,  ESCS,
                 JOYR, DIVR, MEMO, ELAB, CSTR), RCOM, strict=FALSE, OR=20)
detach()

plot(bma,include.intercept=FALSE)
summary(bma)

# bma$label
bma.models = data.frame(cbind(PMP=round(bma$postprob,2), BIC=round(bma$bic,2), Label=bma$label))
Models <- paste0("M", 1:4)
bma.models <- cbind(Models, bma.models)
write.csv(bma.models, file="bmaModels.csv", row.names=F)

# str(bma)
mean=bma$postmean
sd=bma$postsd
prob=c(100.0,bma$probne0)
bma.out <- data.frame(round(cbind(mean, sd, prob), 2))
write.csv(bma.out, file="bmaResult.csv", row.names=F)

#---------B-Model(the best model in BMA)-------------#
bma1.label <- bma$label[1]
nvar <- nchar(bma1.label)/4
var <- rep(NA, nvar)
for (i in 1: nvar)
{
  j = 4 * (i-1) + 1
  var[i] <- substr(bma1.label, j, j+3)
}
data.lm <- PISA09Reg.MAD[, c("RCOM", var)]
names(data.lm)
b.lm <- lm(RCOM~., data=data.lm)

#---------Bayesian Regression-------------#
FullModel <- MCMCregress(RCOM~GEND+NATI+SLAN+ESCS+
                           JOYR+ DIVR+ MEMO+ ELAB+ CSTR,
                         data=PISA09Reg.MAD,burnin=5000,mcmc=100000,thin=10,b0=0,B0=0)

pdf("D:/PsychM/Reg_PMBMA_FullModel.pdf")
plot(FullModel)
dev.off()

pdf("D:/PsychM/Reg_PMBMA_autocorr.pdf")
autocorr.plot(FullModel)
dev.off()

# Diagnostics
sink("Diagnostics.out")
geweke.diag(FullModel, frac1=0.1, frac2=0.5)
heidel.diag(FullModel, eps=0.1, pvalue=0.05)
raftery.diag(FullModel, q=0.5, r=0.05, s=0.95, converge.eps=0.001)
sink()


# Save the result of the Bayesian Regression
sink("MCMCsummary.out")
summary(FullModel)#Posterior Mean, posterior SD and posterior probablity interval (PPI) 
#Note that tau1, tau2 and tau 3 are error precisions instead of variances
sink()

#---------Frequentist : Regression model -------------#
s.lm <- lm(RCOM~., data=PISA09Reg.MAD)

sink("Freq_Regression.out")
summary(s.lm)
sink()


##############################################################
# Predictive performance for the test data--------------
##BMA
predictors.new <- as.matrix(PISA09Reg.PTD[, -1], nrow=nrow(PISA09Reg.PTD) )
bma.predict = predict(bma, newdata=predictors.new,quantiles=c(.05, .95))
bma.lwr <-bma.predict$quantile[,1]
bma.upr <-bma.predict$quantile[,2]
bma.y.new<-PISA09Reg.PTD[, 1]
bma.coverage <- 1- sum(bma.y.new < bma.lwr | bma.y.new > bma.upr) / length(bma.y.new)

##B-Model(the best model in BMA)
lm.predict <- predict(b.lm, newdata=PISA09Reg.PTD, interval="prediction", level=.90)
lm.lwr = lm.predict[,2 ]
lm.upr = lm.predict[,3 ]
lm.y.new = PISA09Reg.PTD[,1]
best.coverage <- 1- sum(lm.y.new < lm.lwr | lm.y.new > lm.upr) / length(lm.y.new)

##Bayesian 
FullModel.coefs <- apply(FullModel,2,median)[1:10]
# FullModel.se = sqrt(5862.759)
# FullModel.pred = predict.PMBMA(coef.vec=FullModel.coefs, se=FullModel.se, newdata=PISA09Reg.PTD )
FullModel.pred = predict.PMBMA(coef.vec=FullModel.coefs, newdata=PISA09Reg.PTD )
lm.lwr = FullModel.pred$lwr
lm.upr = FullModel.pred$upr
lm.y.new = PISA09Reg.PTD[,1]
bayesian.coverage <- 1- sum(lm.y.new < lm.lwr | lm.y.new > lm.upr) / length(lm.y.new)

##S-Model(the selected model)
# lm.predict <- predict(s.lm, newdata=PISA09Reg.PTD, interval="prediction", level=.90)
# lm.lwr = lm.predict[,2 ]
# lm.upr = lm.predict[,3 ]
# lm.y.new = PISA09Reg.PTD[,1]
# freq.coverage <- 1- sum(lm.y.new < lm.lwr | lm.y.new > lm.upr) / length(lm.y.new)
# [1] 0.9045491

s.lm.coefs <- coef(s.lm)
# s.lm.se = summary(s.lm)$sigma
# s.lm.pred = predict.PMBMA(coef.vec=s.lm.coefs, se=s.lm.se, newdata=PISA09Reg.PTD )
s.lm.pred = predict.PMBMA(coef.vec=s.lm.coefs, newdata=PISA09Reg.PTD )
lm.lwr = s.lm.pred$lwr
lm.upr = s.lm.pred$upr
lm.y.new = PISA09Reg.PTD[,1]
freq.coverage <- 1- sum(lm.y.new < lm.lwr | lm.y.new > lm.upr) / length(lm.y.new)
# [1] 0.9045491


sink("P_Coverage_Fun.out")
print(paste("BMA", round(bma.coverage*100,2), round(log(bma.coverage),2), sep=" : "))
print(paste("Best", round(best.coverage*100,2),round(log(best.coverage),2), sep=" : "))
print(paste("Bayesian", round(bayesian.coverage*100,2),round(log(bayesian.coverage),2), sep=" : "))
print(paste("Frequentist", round(freq.coverage*100,2),round(log(freq.coverage),2), sep=" : "))
sink()


#############################################################
#Using my function
#############################################################
##BMA
summary(bma)
str(bma)

bma$postmean
var=bma$residvar
pmp=bma$postprob

#weigthed se by pmp 
bma.se=sqrt(sum(pmp*var))
bma.coefs <- bma$postmean
bma.pred = predict.PMBMA(coef.vec=bma.coefs, se=bma.se, newdata=PISA09Reg.PTD )

bma.lwr <-bma.pred$lwr
bma.upr <-bma.pred$upr
bma.y.new<-PISA09Reg.PTD[, 1]
bma.coverage <- 1- sum(bma.y.new < bma.lwr | bma.y.new > bma.upr) / length(bma.y.new)
# [1] 0.9045491 # this is too low compared to the built-in function (predict)


## couldn't get SE term for the model 
# > predict.bicreg
# Error: object 'predict.bicreg' not found


### S-Model
s.lm.coefs <- coef(s.lm)
s.lm.se = summary(s.lm)$sigma
s.lm.pred = predict.PMBMA(coef.vec=s.lm.coefs, se=s.lm.se, newdata=PISA09Reg.PTD )

lm.lwr = s.lm.pred$lwr
lm.upr = s.lm.pred$upr
lm.y.new = PISA09Reg.PTD[,1]
freq.coverage <- 1- sum(lm.y.new < lm.lwr | lm.y.new > lm.upr) / length(lm.y.new)





# Run Regression model ------------------------------------------
s.lm <- lm(RCOM~., data=PISA09Reg.MAD)
summary(s.lm)


s.lm.pred0 = predict(s.lm, newdata=PISA09Reg.PTD, interval="prediction", level=.90)
head(s.lm.pred0)


## call predict.PMBMA
### multiple linear regression
summary(s.lm)
# Call:
#   lm(formula = RCOM ~ ., data = PISA09Reg.MAD)
# 
# Residuals:
#   Min      1Q  Median      3Q     Max 
# -369.17  -50.41    1.98   55.05  310.79 
# 
# Coefficients:
#   Estimate Std. Error t value Pr(>|t|)    
# (Intercept)  486.969      4.802 101.417   <2e-16 ***
#   GEND           4.286      3.318   1.291   0.1967    
# NATI          -2.145      5.512  -0.389   0.6972    
# SLAN           8.547      6.472   1.321   0.1867    
# ESCS          28.357      1.848  15.346   <2e-16 ***
#   JOYR          30.083      1.796  16.750   <2e-16 ***
#   DIVR          -3.409      1.687  -2.021   0.0434 *  
#   MEMO         -21.012      1.865 -11.264   <2e-16 ***
#   ELAB         -15.649      1.796  -8.713   <2e-16 ***
#   CSTR          28.524      2.110  13.516   <2e-16 ***
#   ---
#   Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1
# 
# Residual standard error: 76.54 on 2452 degrees of freedom
# Multiple R-squared:  0.3455,  Adjusted R-squared:  0.3431 
# F-statistic: 143.8 on 9 and 2452 DF,  p-value: < 2.2e-16

### Predict Interval
s.lm.coefs <- coef(s.lm)
s.lm.se = summary(s.lm)$sigma
s.lm.pred = predict.PMBMA(coef.vec=s.lm.coefs, se=s.lm.se, newdata=PISA09Reg.PTD )

str(s.lm.pred)
head(s.lm.pred$fit)
head(s.lm.pred$lwr)
head(s.lm.pred$upr)

## Predict Interval
summary(FullModel)
FullModel.coefs <- apply(FullModel,2,median)[1:10]
FullModel.se = sqrt(5862.759)
FullModel.pred = predict.PMBMA(coef.vec=FullModel.coefs, se=FullModel.se, newdata=PISA09Reg.PTD )

head(FullModel.pred$fit)
head(FullModel.pred$lwr)
head(FullModel.pred$upr)